#ifndef MPPDECODER_H
#define MPPDECODER_H

#include <rockchip/rk_mpi.h>
#include <rockchip/mpp_buffer.h>
#include <rockchip/mpp_frame.h>
#include <rockchip/mpp_packet.h>
#include <rockchip/rk_type.h>
#include <rockchip/rk_mpi_cmd.h>

#define MPP_ALIGN(x, a) (((x) + (a) - 1) & ~((a) - 1))

class MppDecoder
{
public:
    static MppDecoder &getInstance()
    {
        static MppDecoder instance;
        return instance;
    }

    MppDecoder(const MppDecoder &) = delete;
    MppDecoder &operator=(const MppDecoder &) = delete;

    bool init();
    bool decodeJpegToRgb(const char *jpegData, size_t jpegSize, unsigned char **rgbData, int *width, int *height);

private:
    MppDecoder() = default;
    ~MppDecoder() = default;

    bool readMemToBuffer(const char *data, size_t size, MppBuffer &buffer);

private:
    MppCtx m_ctx{nullptr};
    MppApi *m_mpi{nullptr};
    MppDecCfg m_decCfg{nullptr};
    uint32_t m_needSplit{0};
    MppBufferGroup m_PacketGroup{nullptr};
    MppBufferGroup m_frameGroup{nullptr};
    MppFrame m_frame{nullptr};
    MppPacket m_packet{nullptr};
    RK_U32 m_width{3840};
    RK_U32 m_height{2160};
    RK_U32 m_format;

    MppBuffer m_packetBuffer{nullptr};
    MppBuffer m_frameBuffer{nullptr};
    void *m_inputBufferPtr{nullptr};

    MppTask m_task{nullptr};

    unsigned char *m_rgbData{nullptr};
};

#endif // MPPDECODER_H