#include "RgaColorTransfer.h"

RgaColorTransfer::RgaColorTransfer()
{
    m_Src = new RGA_BUFFER;
    m_Dst = new RGA_BUFFER;
}

RgaColorTransfer::~RgaColorTransfer()
{
    ReleaseResource();
}

bool RgaColorTransfer::copyBuffer(cv::Rect srcRect, cv::Rect dstRect)
{
    if (!m_Src || !m_Dst)
        return false;

    int ret = 0;

    im_rect srcRectRga = { srcRect.x, srcRect.y, srcRect.width, srcRect.height };
    im_rect dstRectRga = { dstRect.x, dstRect.y, dstRect.width, dstRect.height };

    ret = imcheck(&m_Src->SrcImg, &srcRectRga, &m_Dst->SrcImg, &dstRectRga);
    if (ret != 0)
    {
        printf("imcheck failed, ret = %d\n", ret);
        return false;
    }

    // Use improcess to perform the copy operation between the source and destination buffers.
    ret = improcess(m_Src->SrcImg,m_Dst->SrcImg, {}, &srcRectRga, &dstRectRga, NULL,IM_ASYNC);
    
    if (ret != 0)
    {
        printf("improcess failed, ret = %d\n", ret);
        return false;
    }
    return true;
}

bool RgaColorTransfer::transfer(int nType)
{
    if (!m_Src || !m_Dst)
        return false;

    int ret = 0;

    ret = imcvtcolor(m_Src->SrcImg, m_Dst->SrcImg,m_Src->nType,m_Dst->nType);

    if (ret != 0)
    {
        printf("imcvtcolor failed, ret = %d\n", ret);
        return false;
    }

    return true;

}

bool RgaColorTransfer::resizeImage(cv::Rect srcRect, cv::Rect dstRect)
{
    if(!m_Src || !m_Dst)
        return false;
    
    if(m_Src->nWidth == 0 || m_Src->nHeight == 0 || m_Dst->nWidth == 0 || m_Dst->nHeight == 0)
        return false;

    if(m_Src->nWidth == m_Dst->nWidth && m_Src->nHeight == m_Dst->nHeight)
        return true;

    int ret = 0;

    double fx = (double)dstRect.width / (double)srcRect.width;
    double fy = (double)dstRect.height / (double)srcRect.height;

    ret = imresize(m_Src->SrcImg, m_Dst->SrcImg,fx,fy,INTER_LINEAR,IM_SYNC);
    if(ret != 0)
    {
        printf("imresize failed, ret = %d\n", ret);
        return false;
    }
    return true;
}

void RgaColorTransfer::ReleaseResource()
{
    if (m_Src)
    {
        m_Src->ReleaseResource();
        delete m_Src;
        m_Src = nullptr;
    }

    if (m_Dst)
    {
        m_Dst->ReleaseResource();
        delete m_Dst;
        m_Dst = nullptr;
    }
}
