#include "InfraredManager.hpp"
#include <thread>


void InfraredManager::init(std::string infraredStreamPath)
{
    m_infraredStreamPath = infraredStreamPath;
    openStream();
}

void InfraredManager::openStream()
{
    m_infraredStream.open(m_infraredStreamPath);
    if (!m_infraredStream.isOpened())
    {
        std::cerr << "Error: cannot open the infrared stream!" << std::endl;
        exit(1);
    }
    m_infraredStreamWidth = m_infraredStream.get(cv::CAP_PROP_FRAME_WIDTH);
    m_infraredStreamHeight = m_infraredStream.get(cv::CAP_PROP_FRAME_HEIGHT);
    m_infraredStreamFps = m_infraredStream.get(cv::CAP_PROP_FPS);

    m_getInfraredImageThreadSwitch = false;
    std::thread(getInfraredImageThread,this);

}

void InfraredManager::getInfraredImageThread(InfraredManager* infraredManager)
{
    infraredManager->m_getInfraredImageThreadSwitch = true;
    static int index = 0;
    while (infraredManager->m_getInfraredImageThreadSwitch)
    {
        char savePath[254];
        std::lock_guard<std::mutex> lock(infraredManager->m_mutex);
        cv::Mat infraredImage;
        infraredManager->m_infraredStream >> infraredManager->m_infraredImage;
        //save to path
        sprintf(savePath, "./data/infrared_%d.jpg", index++);
        cv::imwrite(savePath, infraredManager->m_infraredImage);
    }
}
