#include "../libuvc/libuvc.h"
#include "../MppDecoder/MppDecoder.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include "UVCCallBack.h"
#include <list>
#include "UVCDevice.h"
#include <iostream>
#include <memory>
#include "../gpio_explorer.hpp"

class UVCManager
{
public:
    static UVCManager &getInstance()
    {
        static UVCManager instance;
        return instance;
    }

    void init()
    {
        // get device list
        uvc_device_t **deviceList;
        uvc_get_device_list(ctx, &deviceList);
        // get device number
        deviceNum = 0;
        while (deviceList[deviceNum] != NULL)
        {
            deviceNum++;
        }
        std::cout << "Device Number :" << deviceNum << std::endl;
        // init the Uvc device by deviceNum
        for (int i = 0; i < deviceNum; i++)
        {
            UVCDevice *uvcDevice = new UVCDevice();
            uvcDevice->init(i, ctx, deviceList[i]);
            uvcDevice->setCallBack(cbSaveToLocal);
            uvcDeviceList.push_back(uvcDevice);
        }

        if (MppDecoder::getInstance().init())
        {
            std::cout << "init MppDecoder success" << std::endl;
        }

        m_GpioExplorer = std::make_shared<GPIOExplorer>("gpiochip0", 16);
    }

    int deliverFrameInCallBack(uvc_device_handle_t *devh)
    {
        int i = 0;
        std::list<UVCDevice *>::iterator it = uvcDeviceList.begin();
        for (; it != uvcDeviceList.end(); it++, i++)
        {
            if ((*it)->getDevh() == devh)
            {
                return i;
            }
        }
        return -1;
    }

    void startAllStreaming()
    {
        for (auto uvcDevice : uvcDeviceList)
        {
            uvcDevice->startStreaming();
        }
    }

    void stopAllStreaming()
    {
        for (auto uvcDevice : uvcDeviceList)
        {
            uvcDevice->stopStreaming();
        }
    }

    uvc_context_t *getContext()
    {
        return ctx;
    }

private:
    UVCManager()
    {
        uvc_init(&ctx, NULL);
    }

    ~UVCManager()
    {
        uvc_exit(ctx);
    }

    UVCManager(const UVCManager &) = delete;
    UVCManager &operator=(const UVCManager &) = delete;

    uvc_context_t *ctx;
    // device list
    uvc_device_t **deviceList;
    // device number
    int deviceNum;

    // the UVC device list is been managed by UVCManager
    std::list<UVCDevice *> uvcDeviceList;

    //
    std::shared_ptr<GPIOExplorer> m_GpioExplorer;
};